<?php
/**
 * Plugin Name: Serial codes generator and validator
 * Plugin URI: https://nikolov.org/wp-plugins/sngmbh-serial-codes-validator/
 * Description: You can create and generate serials and codes. Print them on your products boxes or manuals. Your customer can check if the code is valid on your website. The Premium allows you also to activate user registration and more. This allows your user to register them self to a serial number.
 * Version: 1.0.4
 * Author: Saso Nikolov
 * Author URI: https://nikolov.org/wp-plugins
 * Text Domain: sngmbh-serial-codes-validator
 *
 * Serial Code Validator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */
defined('ABSPATH') or die('Direct access not allowed');

if (!defined('SNGMBH_SERIALCODES_VALIDATOR_PLUGIN_VERSION')) define('SNGMBH_SERIALCODES_VALIDATOR_PLUGIN_VERSION', '1.0.4');
if (is_admin() && isset($_REQUEST['SngmbhSerialcodesValidatorDebug'])) {
	defined('WP_DEBUG') or define( 'WP_DEBUG', true );
	ini_set('display_startup_errors', 'On');
	error_reporting(2147483647); // max future error values
	ini_set('display_errors', 'On');
}
class sngmbhSerialcodesValidator_base {
	private $_premFunctions;
	private $_isPrem = false;
	private $_maxValues = ['codes'=>1000,'codes_total'=>1000,'lists'=>5,'storeip'=>false,'allowuserreg'=>false];

	public function __construct($_prefix) {
		$this->init($_prefix);
	}
	private function init($_prefix) {
		$file = $this->getPremiumPluginFolder()."sngmbh-serialcodes-validator-functions.php";
		$premiumFile = plugin_dir_path(__FILE__)."../".$file;
		if (file_exists($premiumFile) && is_plugin_active(dirname($file)."/".dirname($file).".php")) {
			include_once $premiumFile;
			$this->_premFunctions = new sngmbhSerialcodesValidator_PremiumFunctions(plugin_dir_path(__FILE__), $_prefix);
			$this->_isPrem = $this->_premFunctions->isPremium();
			if ($this->_isPrem) $this->_maxValues = $this->_premFunctions->maxValues();
		}
	}
	public function getPremiumPluginFolder() {
		return basename(dirname(__FILE__))."-premium/";
	}
	public function getPremiumFunctions() {
		return $this->_premFunctions;
	}
	public function isPremium() {
		return $this->_isPrem;
	}
	public function getMaxValues() {
		return $this->_maxValues;
	}
	public function premiumCheck_isAllowedAddingList($total) {
		if ($this->_maxValues['lists'] == 0) return true;
		if ($total > $this->_maxValues['lists']) return false;
		return true;
	}
	public function premiumCheck_isAllowedAddingCode($total) {
		if ($this->_maxValues['codes_total'] == 0) return true;
		if ($total > $this->_maxValues['codes_total']) return false;
		return true;
	}
}
class sngmbhSerialcodesValidator {
	private $_base;
	private $_js_version = SNGMBH_SERIALCODES_VALIDATOR_PLUGIN_VERSION;
	private $_js_file = 'sngmbh-serialcodes-validator.js';
	private $_js_nonce = 'sngmbhSerialcodesValidator';
	protected $_prefix = 'sngmbhSerialcodesValidator';
	protected $_shortcode = 'sngmbhSerialcodesValidator';
	protected $_divId = 'sngmbhSerialcodesValidator';

	public function __construct() {
		if (defined( 'WP_DEBUG')) $this->_js_version = time();
		if (is_admin()) { // called in backend admin, admin-ajax!
			$this->init_backend();
		} else { // called in front end
			$this->init_frontend();
		}
  		if (basename($_SERVER['SCRIPT_NAME']) == "admin-ajax.php") {
			add_action('wp_ajax_nopriv_'.$this->_prefix.'_executeFrontend', [$this,'executeFrontend']); // nicht angemeldete user, sollen eine antwort erhalten
			add_action('wp_ajax_'.$this->_prefix.'_executeFrontend', [$this,'executeFrontend']); // falls eingeloggt ist
		}
	}
	public function getBase() {
		if ($this->_base == null) $this->_base = new sngmbhSerialcodesValidator_base($this->_prefix);
		return $this->_base;
	}
	private function init_frontend() {
		add_shortcode($this->_shortcode, [$this, 'replaceShortcode']);
	}
	private function init_backend() {
		//add_action('admin_init', [$this, 'initialize_plugin']);
		add_action('admin_menu', [$this, 'register_options_page']);
		register_activation_hook(__FILE__, [$this, 'plugin_activated']);
		//register_deactivation_hook( __FILE__, [$this, 'plugin_deactivated'] );
		register_uninstall_hook( __FILE__, [$this, 'plugin_uninstall'] );
		add_action( 'plugins_loaded', [$this, 'plugins_loaded'] );

		if (basename($_SERVER['SCRIPT_NAME']) == "admin-ajax.php") {
			add_action('wp_ajax_'.$this->_prefix.'_executeAdminSettings', [$this,'executeAdminSettings']);
		}
	}
	public function plugin_uninstall(){
    	include_once plugin_dir_path(__FILE__)."db.php";
    	$db = new sngmbhSerialcodesValidatorDB();
    	$db->plugin_uninstall();

		include_once plugin_dir_path(__FILE__)."sngmbhSerialcodesValidator_AdminSettings.php";
		$sngmbhSerialcodesValidator_AdminSettings = new sngmbhSerialcodesValidator_AdminSettings($this->getBase());
		$sngmbhSerialcodesValidator_AdminSettings->plugin_uninstall();
	}
	public function plugin_activated($is_network_wide=false) { // und auch für updates, macht es einfacher
    	include_once plugin_dir_path(__FILE__)."db.php";
    	$db = new sngmbhSerialcodesValidatorDB();
    	$db->installiereTabellen(); // schützt siche selbst mit eigener option-var
    	update_option('SNGMBH_SERIALCODES_VALIDATOR_PLUGIN_VERSION', SNGMBH_SERIALCODES_VALIDATOR_PLUGIN_VERSION);
	}
	public function plugins_loaded() {
		if (SNGMBH_SERIALCODES_VALIDATOR_PLUGIN_VERSION !== get_option('SNGMBH_SERIALCODES_VALIDATOR_PLUGIN_VERSION')) $this->plugin_activated(); // vermutlich wurde die aktivierung übersprungen, bei änderungen direkt an den files
	}
    public function initialize_plugin() {
    	include plugin_dir_path(__FILE__)."db.php";
    	$db = new sngmbhSerialcodesValidatorDB();
		$db->installiereTabellen();
    }

	function register_options_page() {
	  	add_options_page('Serial Codes Validator', 'Serial Codes Validator', 'manage_options', 'sngmbh-serialcodes-validator', [$this,'options_page']);
	}

	function options_page() {
		if ( !current_user_can( 'manage_options' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}

    	include_once plugin_dir_path(__FILE__)."db.php";
    	$db = new sngmbhSerialcodesValidatorDB();
    	$db->installiereTabellen(); // check DB und repair falls nötig

		// einbinden das js starter skript
		$js_url = $this->_js_file."?_v=".$this->_js_version;
		if (defined( 'WP_DEBUG')) $js_url .= '&debug=1';

        wp_enqueue_script(
            'ajax_script',
            plugins_url( $js_url,__FILE__ ),
            array('jquery', 'jquery-ui-dialog')
        );

		// per script eine variable einbinden, die url hat den wp-admin prefix
		// damit im backend.js dann die richtige callback url genutzt werden kann
        wp_localize_script(
            'ajax_script',
            'Ajax_'.$this->_prefix, // name der injected variable
            array(
            	'_plugin_home_url' =>plugins_url( "",__FILE__ ),
            	'_action' => $this->_prefix.'_executeAdminSettings',
            	'_max'=>$this->getBase()->getMaxValues(),
            	'_isPremium'=>$this->getBase()->isPremium(),
            	'_premJS'=>$this->getBase()->isPremium() ? $this->getBase()->getPremiumFunctions()->getJSBackendFile() : '',
                'url'   => admin_url( 'admin-ajax.php' ),
                'nonce' => wp_create_nonce( $this->_js_nonce ),
                'ajaxActionPrefix' => $this->_prefix,
                'divPrefix' => $this->_prefix,
                'divId' => $this->_divId,
                'jsFiles' => plugins_url( 'backend.js',__FILE__ )
            )
        );

		?>
		<div>
			<h2>Serial Codes Validator</h2>
			<p>Shortcode to display the serial check for the user: <b>[<?php echo $this->_shortcode; ?>]</b></p>
			<div id="<?php echo $this->_divId; ?>"></div>
	  	</div>
		<?php
	}

	public function executeAdminSettings() {
		include_once plugin_dir_path(__FILE__)."sngmbhSerialcodesValidator_AdminSettings.php";
		$sngmbhSerialcodesValidator_AdminSettings = new sngmbhSerialcodesValidator_AdminSettings($this->getBase());
		if (!isset($_REQUEST['a'])) return wp_send_json_success("a not provided");
		$data = isset($_REQUEST['data']) ? $_REQUEST['data'] : [];
		return $sngmbhSerialcodesValidator_AdminSettings->executeJSON($_REQUEST['a'], $data);
	}

	public function executeFrontend() {
		include_once plugin_dir_path(__FILE__)."sngmbhSerialcodesValidator_Frontend.php";
		$sngmbhSerialcodesValidator_Frontend = new sngmbhSerialcodesValidator_Frontend($this->getBase());
		if (!isset($_REQUEST['a'])) return wp_send_json_success("a not provided");
		$data = isset($_REQUEST['data']) ? $_REQUEST['data'] : [];
		return $sngmbhSerialcodesValidator_Frontend->executeJSON($_REQUEST['a'], $data);
	}

	public function replaceShortcode($attr=[]) {
		// einbinden das js starter skript
		$js_url = $this->_js_file."?_v=".$this->_js_version;
		if (defined( 'WP_DEBUG')) $js_url .= '&debug=1';

      	wp_enqueue_script(
            'ajax_script',
            plugins_url( $js_url,__FILE__ ),
            array('jquery')
        );

		$vars = array(
            	'_plugin_home_url' =>plugins_url( "",__FILE__ ),
            	'_action' => $this->_prefix.'_executeFrontend',
            	'_isPremium'=>$this->getBase()->isPremium(),
            	'_premJS'=>$this->getBase()->isPremium() ? $this->getBase()->getPremiumFunctions()->getJSFrontFile() : '',
                'url'   => admin_url( 'admin-ajax.php' ),
                'nonce' => wp_create_nonce( $this->_js_nonce ),
                'ajaxActionPrefix' => $this->_prefix,
                'divPrefix' => $this->_prefix,
                'divId' => $this->_divId,
                'jsFiles' => plugins_url( 'validator.js',__FILE__ )
            );
		if ($this->getBase()->isPremium()) {
			$vars['_messages'] = [
				'msgCheck0'=>$this->getBase()->getPremiumFunctions()->getOption('textValidationMessage0')['value'],
				'msgCheck1'=>$this->getBase()->getPremiumFunctions()->getOption('textValidationMessage1')['value'],
				'msgCheck2'=>$this->getBase()->getPremiumFunctions()->getOption('textValidationMessage2')['value'],
				'msgCheck3'=>$this->getBase()->getPremiumFunctions()->getOption('textValidationMessage3')['value'],
				'msgCheck4'=>$this->getBase()->getPremiumFunctions()->getOption('textValidationMessage4')['value'],
				'msgCheck5'=>$this->getBase()->getPremiumFunctions()->getOption('textValidationMessage5')['value']
			];
		}

		if ($this->getBase()->isPremium()) $this->getBase()->getPremiumFunctions()->addJSFrontFile();

		// per script eine variable einbinden, die url hat den wp-admin prefix
		// damit im backend.js dann die richtige callback url genutzt werden kann
        wp_localize_script(
            'ajax_script',
            'Ajax_'.$this->_prefix, // name der injected variable
            $vars
        );
		return '<div id="'.$this->_divId.'">...loading...</div>';
	}
}
$sngmbhSerialcodesValidator = new sngmbhSerialcodesValidator();
?>